#include "WaveformDisplay.h"

#include <stdio.h>


BEGIN_EVENT_TABLE(WaveformDisplay,wxGLCanvas)
    EVT_PAINT(WaveformDisplay::OnPaint)
    EVT_SIZE(WaveformDisplay::OnSize)
END_EVENT_TABLE()


int SignalColors[MAX_CHANNELS][3] = {
    
    {237, 164,  61},        // Crimson
    {255,   0,   0},        // Red
    {  0, 191, 255},        // Deep sky blue
    {255, 255,   0},        // Yellow
    {  0, 255, 255},        // Aqua   
    {238, 130, 238},        // Violet
    {255, 165,   0},        // Orange
    {152, 251, 152},        // Pale Green
    {218, 165,  32},        // Goldenrod
    {148,   0, 211},        // Dark Violet
    {210, 105,  30},        // Chocolate
    {139,   0,   0},        // Dark Red
    {154, 205,  50},        // Yellow-Green
    {255, 255, 255},        // White
    {128, 128, 128},        // Gray
    {244, 164,  96},        // Sienna Brown
//    {  0, 128,   0},        // Green
    
};

WaveformDisplay::WaveformDisplay(wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size, wxString desc) 
          :wxGLCanvas(parent,id,pos,size) {

    for(int i=0; i < MAX_CHANNELS; i++) {
        scalingFactor[i] = 1;   
    }
    
    BufferSize = 0;
    ActiveChannels = 0x0001;
    Buffer = NULL;
}

WaveformDisplay::~WaveformDisplay() {
    
}

void WaveformDisplay::SetUpGL() {
    wxSize temp = this->GetSize();
    glViewport(0,0,temp.GetWidth(),temp.GetHeight());
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(-1.5,20.5,-7.5,6.5,0.0f,100.0f);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glClearColor(0.05f, 0.25f, 0.05f, 0.5f);
    glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);
}

int WaveformDisplay::SetBufferSize(int newSize) {
    BufferSize = newSize;
    return BufferSize;
}

int WaveformDisplay::SetActiveChannels(int newActiveChannels) {
    ActiveChannels = newActiveChannels;
    
    int count = 0;
    
    while (newActiveChannels != 0) {
        count++;
        newActiveChannels = newActiveChannels >> 1;   
    }
    
    return count;
}

void WaveformDisplay::SetBuffer(float* newBuffer) {
    Buffer = newBuffer;
}

void WaveformDisplay::SetOffset(int ch, float offset) {
    if (ch < 0 || ch > MAX_CHANNELS)
        return;
        
    Offsets[ch] = offset;
}

void WaveformDisplay::Render() {
    wxClientDC dc(this);
    Draw(dc);
}

void WaveformDisplay::OnPaint(wxPaintEvent &event) {
    wxPaintDC dc(this);
    Draw(dc);    
}

void WaveformDisplay::Draw(wxDC &dc) {
    SetCurrent();

    glClearColor(0.0f, 0.0f, 0.00f, 0.5f);
    glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT);
    SetUpGL();
    try{
	SetCurrent();

    RenderBackground();
    RenderSignal();


    }catch(...) {
    }
    
    glFinish();
    try {
      SwapBuffers();
    } catch (...) {
      printf("exception thrown\n");
    }
    dc.SetTextForeground(wxColour(255,255,255) );
    dc.SetFont(wxFont(10, wxFONTFAMILY_SWISS, wxFONTWEIGHT_NORMAL, false ) );
    dc.DrawText("1.20 V", 2, 5);
    dc.DrawText("0.0 V", 10, 325);
    dc.DrawText("0 ps", 30, 345);
    dc.DrawText(wxString::Format("%.1f ns", timeBase), 580, 350);
}

void WaveformDisplay::OnSize(wxSizeEvent &event) {
    this->SetSize(event.GetSize());
}

void WaveformDisplay::RenderBackground() {
    // horizontal lines
    for(int i=-6; i <= 6; i++ ) {
        if (i == 0)
            glLineWidth(2.0f);
        else
            glLineWidth(1.0f);

        glBegin(GL_LINE_STRIP);

        glColor3f(0.2f,0.8f,0.2f);            

        glVertex2f(0, (float)i);
        glVertex2f(20, (float)i);
        glEnd();
    }

    // vertical lines
    for(int i=0; i <= 20; i++ ) {
        glBegin(GL_LINE_STRIP);
        glColor3f(0.2f,0.8f,0.2f);
        glVertex2f((float)i, -6);
        glVertex2f((float)i, 6);
        glEnd();
    }
    
}

void WaveformDisplay::RenderSignal() {
    if (ActiveChannels == 0 || BufferSize < 2 || Buffer == NULL)
        return;
    
    for(int i=0; i < MAX_CHANNELS; i++) {
        float x, y;

        if ( !(ActiveChannels & (1 << i)))
            continue;

        glBegin(GL_LINE_STRIP);
        glColor3f(SignalColors[i][0] / 255.0, SignalColors[i][1] / 255.0, SignalColors[i][2] / 255.0);
        for(int j=0; j < BufferSize; j++) {
            // x is just an interpolation from 0 to 20, based on position in buffer
            x = 20.0 * j / (BufferSize - 1);
            
            // y is the y-value; later affected by offset and scaling
            y = Buffer[i * BufferSize + j];  // the value
            y /= scalingFactor[i];              // scale it to vertical scaling
            y += Offsets[i];                 // vertical offset based on channel
        
            glVertex2f(x, y);
        }
        glEnd();
    }
}

float WaveformDisplay::SetScale(float newScale, int ch) {
    if (ch < 0 || ch >= MAX_CHANNELS) return 0;
    scalingFactor[ch] = newScale;
    return newScale;
}
